bl_info = {
    "name": "MaSzyna T3D model exporter",
    "description": "Addon for exporting models to MaSzyna T3D text format",
    "author": "Krzysztof Przybyla (krzysiuup), Grzegorz Firlejczyk (firleju)",
    "version": (1, 0, 0),
    "blender": (2, 78, 0),
    "location": "File > Export",
    "wiki_url": "https://wiki.eu07.pl/index.php/Blender_-_modelowanie",
    "support": "OFFICIAL",
    "category": "Import-Export",
}


import bpy
from bpy.props import StringProperty, BoolProperty, FloatProperty, EnumProperty
from bpy_extras.io_utils import ExportHelper

from maszyna_utils.io.t3d import T3DDumper

from .access_layers import get_access_layer
from .build_context import build_context


class ExportT3D(bpy.types.Operator, ExportHelper):
    """ Save model in MaSzyna Train Simulator T3D text format """
    bl_idname = "export_model.t3d"
    bl_label = "Export T3D"
    filename_ext = ".t3d"

    filter_glob = StringProperty(
        default="*.t3d",
        options={'HIDDEN'}
    )
    use_selection = BoolProperty(
        name="Selection Only",
        description="Export only selected objects",
        default=False
    )
    use_modifiers = BoolProperty(
        name="Apply modifiers",
        description="Apply modifiers",
        default=True
    )
    bezier_to_mesh = BoolProperty(
        name='Bezier to Mesh',
        description='Save beziers as meshes',
        default=False
    )
    use_explicit_normals = BoolProperty(
        name='Explicit normals',
        description='Save normal vectors to output file',
        default=False
    )
    use_pseudo_dds_flip = BoolProperty(
        name='Pseudo DDS flip',
        description='Flip UV mapping vertically on submodels mapped with DDS textures',
        default=True
    )

    def execute(self, context):
        t3d_tools_addon = context.user_preferences.addons["maszyna_t3d_tools"]
        root_path = t3d_tools_addon.preferences.root_path

        config = {
            'filepath': self.filepath,
            'maszyna_root': root_path,
            'use_modifiers': self.use_modifiers,
            'use_explicit_normals': self.use_explicit_normals,
            'use_pseudo_dds_flip': self.use_pseudo_dds_flip,
        }

        objects = (
            bpy.context.selected_objects if self.use_selection
            else bpy.context.scene.objects
        )

        with T3DDumper(self.filepath) as dumper:

            for object_ in objects:

                try:
                    access_layer = get_access_layer(object_, config)
                except ValueError:
                    continue

                context = build_context(access_layer)

                dumper.dump_submodel(context)

        return {'FINISHED'}


def menu_func_export(self, context):
    self.layout.operator(ExportT3D.bl_idname, text="MaSzyna Text Model (.t3d)")


def register():
    bpy.utils.register_module(__name__)
    bpy.types.INFO_MT_file_export.append(menu_func_export)


def unregister():
    bpy.utils.unregister_module(__name__)
    bpy.types.INFO_MT_file_export.remove(menu_func_export)


if __name__ == '__main__':
    register()
