import abc
import math
from typing import Any, Dict, Iterator, Sequence, Union

import maszyna_utils

import bpy

def get_access_layer(object_: bpy.types.Object, config: Dict[str, Any]):
    if object_.type == 'MESH':
        layer_class = MeshAccessLayer
    elif object_.type == 'LAMP' and object_.data.type == 'SPOT':
        layer_class = FreeSpotLightAccessLayer
    elif object_.type == 'EMPTY':
        layer_class = StarsAccessLayer
    else:
        raise ValueError('Object \"{0}\" of type {1} is unexportable!'.format(
            object_.name, object_.type
        ))

    return layer_class(object_, config)


class SubmodelAccessLayer(abc.ABC):
    """
    Class responsible for translating properties of Blender object to form
    suitable for T3D format.
    """
    def __init__(self, object_: bpy.types.Object, config: Dict[str, Any]):
        self._object = object_
        self._config = config

    def get_parent(self) -> str:
        """ Return name of parent submodel or 'none' when parent does not exists """
        parent = self._object.parent
        return parent.name if parent else 'none'

    @abc.abstractmethod
    def get_type(self) -> str:
        """ Return type name of submodel """
        pass

    def get_name(self) -> str:
        """ Return name of submodel """
        return self._object.name

    def get_anim(self) -> str:
        """ Return animation type of submodel """
        return self._object.t3d_properties.anim

    def get_selfillum(self) -> Union[bool, float]:
        """ Return value of SelfIllum property """
        if self._object.t3d_properties.selfillum_never:
            return False
        elif self._object.t3d_properties.selfillum_always:
            return True
        else:
            return self._object.t3d_properties.selfillum

    def get_max_distance(self) -> float:
        """ Return maximal rendering distance of submodel """
        max_distance = self._object.t3d_properties.max_distance
        max_distance = -1 if max_distance < 0 else max_distance
        return max_distance

    def get_min_distance(self) -> float:
        """ Return minimal rendering distance of submodel """
        return self._object.t3d_properties.min_distance

    def get_transform(self) -> Sequence[Sequence[float]]:
        """ Return transformation matrix of submodel """
        return self._object.matrix_local.transposed()

    def get_num_verts(self):
        """ Return total number of vertices in vert groups """
        return 0

    def get_vertex_groups(self):
        """ Return vertex groups iterator """
        return iter([])


class MeshAccessLayer(SubmodelAccessLayer):
    def __init__(self, object_: bpy.types.Object, config: Dict[str, Any]):
        super().__init__(object_, config)

        self._triangulator = self._object.modifiers.new("T3DTri", 'TRIANGULATE')
        self._mesh = object_.to_mesh(
            bpy.context.scene, self._config['use_modifiers'], 'PREVIEW'
        )

        # QUESTION: Is active one better than first one?
        self._material = self._object.active_material

        self._uv_layer = self._mesh.uv_layers.active

        if self._material:
            self._texture = self._material.active_texture
        else:
            self._texture = None

        if self._texture:
            self._image = self._texture.image
        else:
            self._image = None

    def get_type(self) -> str:
        return 'Mesh'

    def get_ambient(self) -> Sequence[float]:
        """ Return ambient RGB color of submodel """
        return _scale_rgb_to_255(self._object.t3d_properties.ambient_color)

    def get_diffuse(self) -> Sequence[float]:
        """ Return diffuse RGB color of submodel """
        if self._material:
            return _scale_rgb_to_255(self._material.diffuse_color)
        else:
            return [255, 255, 255]

    def get_specular(self) -> Sequence[float]:
        """ Return specular RGB color of submodel """
        if self._material:
            return _scale_rgb_to_255(self._material.specular_color)
        else:
            return [255, 255, 255]

    def get_wire(self) -> bool:
        """ Return wireframe display state of submodel """
        return self._object.t3d_properties.wire

    def get_wire_size(self) -> float:
        """ Return value of wire thickness in wireframe display mode """
        return self._object.t3d_properties.wire_size

    def get_opacity(self) -> int:
        """ Return transparency state of submodel """
        return self._object.t3d_properties.opacity

    def get_map(self) -> str:
        """
        Return replaceable skin ID or relative path to texture. Return 'none',
        if submodel has no texture or replaceable skin.
        """

        replacableskin_id = self._object.t3d_properties.replacableskin_id
        if replacableskin_id != 'None':
            return replacableskin_id
        elif self._image:
            root = bpy.path.abspath(self._config['maszyna_root'])
            model_path = bpy.path.abspath(self._config['filepath'])
            img_path = bpy.path.abspath(self._image.filepath)
            return maszyna_utils.textures.make_relative_path(model_path, img_path, root)
        else:
            return 'none'

    def get_num_verts(self) -> int:
        """ Return vertices count of all triangles """
        return len(self._mesh.polygons) * 3

    # This annotation is shitty af
    def get_vertex_groups(self, **kwargs) -> Iterator[Sequence[Union[int, Sequence[float]]]]:
        smooth_groups = self._mesh.calc_smooth_groups(use_bitflags=True)[0]
        use_explicit_normals = self._config['use_explicit_normals']
        use_pseudo_dds_flip = self._config['use_pseudo_dds_flip']

        for triangle in self._mesh.polygons:

            if use_explicit_normals:
                smooth = -1
            elif triangle.use_smooth:
                smooth = smooth_groups[triangle.index]
            else:
                smooth = 0

            vert_group = [smooth]
            for local_vert_idx, global_vert_idx in enumerate(triangle.vertices):
                vertex = self._mesh.vertices[global_vert_idx]

                coords = vertex.co
                normal = vertex.normal if use_explicit_normals else []
                if self._uv_layer:
                    loop_index = triangle.loop_indices[local_vert_idx]
                    uv = self._uv_layer.data[loop_index].uv

                    if (self._image
                        and self._image.filepath.endswith('.dds')
                        and use_pseudo_dds_flip):
                        uv[1] = self._image.size[1] - uv[1]
                else:
                    uv = [0, 0]

                vertex_array = [*coords, *normal, *uv]
                vert_group.append(vertex_array)

            yield vert_group

        self._object.modifiers.remove(self._triangulator)


class FreeSpotLightAccessLayer(SubmodelAccessLayer):
    def __init__(self, object_: bpy.types.Object, config: Dict[str, Any]):
        super().__init__(object_, config)

    def get_type(self) -> str:
        return 'FreeSpotLight'

    def get_diffuse(self) -> Sequence[float]:
        return _scale_rgb_to_255(self._object.data.color)

    def get_near_atten_start(self) -> float:
        return self._object.t3d_properties.near_atten_start

    def get_near_atten_end(self) -> float:
        return self._object.t3d_properties.near_atten_end

    def get_use_near_atten(self) -> bool:
        return self._object.t3d_properties.use_near_atten

    def get_far_atten_decay_type(self) -> int:
        return self._object.t3d_properties.far_atten_decay_type

    def get_far_decay_radius(self) -> float:
        return self._object.data.distance

    def get_falloff_angle(self) -> float:
        # TODO: Verify it
        spot_size = self._object.data.spot_size
        spot_blend = self._object.data.spot_blend + 1

        return math.degrees(spot_size * spot_blend)

    def get_hotspot_angle(self) -> float:
        # TODO: Verify it
        return math.degrees(self._object.data.spot_size)


class StarsAccessLayer(SubmodelAccessLayer):
    def __init__(self, object_: bpy.types.Object, config: Dict[str, Any]):
        super().__init__(object_, config)
        self._light_points = [
            o for o in object_.children if o.data.type == 'POINT'
        ]

    def get_type(self) -> str:
        return 'Stars'

    def get_diffuse(self) -> Sequence[float]:
        """ Return diffuse color of submodel """
        color = self._object.t3d_properties.diffuse_color
        return [float(value * 255) for value in color]

    def get_num_verts(self) -> int:
        """ Return count of all star points in submodel """
        return len(self._light_points)

    def get_vertex_groups(self):
        MAX_GROUP_SIZE = 3

        num_groups = math.ceil(len(self._light_points) / MAX_GROUP_SIZE)
        for group_no in range(num_groups):
            slice_start = group_no * MAX_GROUP_SIZE
            slice_end = slice_start + MAX_GROUP_SIZE
            group_slice = self._light_points[slice_start:slice_end]

            group = [1]
            for star in group_slice:
                star_array = [
                    *star.location,
                    self._compress_color(star.data.color),
                    0
                ]

                group.append(star_array)

            yield group

# -- Internals --

    def _compress_color(self, color: Sequence[float]) -> str:
        return ((int(color[2] * 255) << 16)
               | (int(color[1] * 255) << 8)
               | int(color[0] * 255))


def _scale_rgb_to_255(color: Sequence[float]):
    return [int(value * 255) for value in color]
