bl_info = {
    'name': 'MaSzyna T3D modeling tools',
    'description': 'Addon for working with T3D models',
    'author': 'Krzysztof Przybyla (krzysiuup)',
    'version': (1, 0, 0),
    'blender': (2, 78, 0),
    'location': 'Properties > T3D',
    'wiki_url': 'https://wiki.eu07.pl/index.php/Blender_-_modelowanie',
    'support': 'OFFICIAL',
    'category': 'User Interface',
}

import bpy
from bpy.props import PointerProperty, StringProperty

from bpy.props import (
    BoolProperty, EnumProperty, FloatProperty, FloatVectorProperty, IntProperty
)


class T3DProperties(bpy.types.PropertyGroup):
    anim = EnumProperty(
        name='Animation type',
        default='false',
        items=[
            ('ik22', 'ik22', ''),
            ('ik21', 'ik21', ''),
            ('ik11', 'ik11', ''),
            ('ik', 'ik', ''),
            ('hours24_jump', 'hours24_jump', ''),
            ('hours24', 'hours24', ''),
            ('hours_jump', 'hours_jump', ''),
            ('hours', 'hours', ''),
            ('minutes_jump', 'minutes_jump', ''),
            ('minutes', 'minutes', ''),
            ('seconds_jump', 'seconds_jump', ''),
            ('seconds', 'seconds', ''),
            ('digital', 'digital', ''),
            ('digiclk', 'digiclk', ''),
            ('billboard', 'billboard', ''),
            ('wind', 'wind', ''),
            ('sky', 'sky', ''),
            ('true', 'true', ''),
            ('false', 'false', '')
        ]
    )

    max_distance = FloatProperty(
        name='Max Distance',
        description='Maximal rendering radius',
        min=-1,
        default=-1,
        step=0.1
    )

    min_distance = FloatProperty(
        name='Min Distance',
        description='Minimal rendering radius',
        min=0,
        step=0.1
    )

    selfillum = FloatProperty(
        name='SelfIllum',
        description='Emit light below the specified light level threshold',
        default=0,
        min=-1.25,
        max=1.25,
        step=0.01
    )

    selfillum_always = BoolProperty(
        name='True',
        description='Emit light always',
        default=False
    )

    selfillum_never = BoolProperty(
        name='False',
        description='Disable light emission',
        default=True
    )

    ambient_color = FloatVectorProperty(
        name='Ambient',
        description='Ambient value of submodel',
        subtype='COLOR',
        default=(1, 1, 1),
        min=0,
        max=1,
    )

    diffuse_color = FloatVectorProperty(
        name='Diffuse',
        description='Diffuse value of submodel',
        subtype='COLOR',
        default=(1, 1, 1),
        min=0,
        max=255
    )

    wire = BoolProperty(
        name='Wire',
        description='Display this mesh as wireframe',
        default=False,
    )

    wire_size = FloatProperty(
        name='Wire size',
        description='Wire thickness in wireframe mode',
        min=0,
        default=1
    )

    opacity = FloatProperty(
        name='Opacity',
        description='Opacity level of submodel',
        min=0,
        max=1,
        default=1
    )

    replacableskin_id = EnumProperty(
        name='Replacableskin ID',
        default='None',
        items=[
            ('None', 'None', ''),
            ('-1', '-1', ''),
            ('-2', '-2', ''),
            ('-3', '-3', ''),
            ('-4', '-4', '')]
    )

    use_near_atten = BoolProperty(
        name='Use near attentuation',
        default=False,
    )

    near_atten_start = FloatProperty(
        name='Near attentuation start distance',
        default=40,
        min=0,
    )

    near_atten_end = FloatProperty(
        name='Near attentuation end distance',
        default=0,
        min=0,
    )

    far_atten_decay_type = IntProperty(
        name='Far attentuation decay type',
        default=1,
        min=0,
        max=2
    )


class T3DPropertiesPanel(bpy.types.Panel):
    bl_idname = "OBJECT_PT_t3d"
    bl_label = "T3D Properties"
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = 'object'

    @classmethod
    def poll(cls, context):
        return context.object

    def draw(self, context):
        self.obj = context.object
        layout = self.layout

        def is_mesh(obj):
            return obj.type == 'MESH'

        def is_spot(obj):
            return obj.type == 'LAMP' and obj.data.type == 'SPOT'

        def is_stars(obj):
            return obj.type == 'EMPTY'

        def is_exportable(obj):
            return is_mesh(obj) or is_spot(obj) or is_stars(obj)

        if is_exportable(self.obj):
            self.draw_common_properties()

            if is_mesh(self.obj):
                self.draw_mesh_properties()
            elif is_spot(self.obj):
                self.draw_spot_properties()
            elif is_stars(self.obj):
                self.draw_stars_properties()

            self.draw_last_common_properties()

            self.layout.row().operator(
                "wm.url_open",
                text="Read the docs [PL]",
                icon='QUESTION'
            ).url = "https://wiki.eu07.pl/index.php/Format_T3D"

        else:
            layout.label('This object is not supported by T3D format!')


    def draw_common_properties(self):
        self.layout.row().prop(
            self.obj, 'name'
        )
        self.layout.row().prop(
            self.obj, 'parent'
        )
        self.layout.row().prop(
            self.obj.t3d_properties, 'anim',
            icon='ANIM',
            text='Anim'
        )
        self.layout.separator()

    def draw_last_common_properties(self):
        self.layout.separator()
        self.layout.row().prop(
            self.obj.t3d_properties, "max_distance",
            text="MaxDistance"
        )
        self.layout.row().prop(
            self.obj.t3d_properties, "min_distance",
            text="MinDistance"
        )

    def draw_mesh_properties(self):
        if self.obj.active_material:
            self.layout.row().prop(
                self.obj.t3d_properties, 'ambient_color',
                text='Ambient'
            )
            self.layout.row().prop(
                self.obj.active_material, 'diffuse_color',
                text='Diffuse'
            )
            self.layout.row().prop(
                self.obj.active_material, 'specular_color',
                text='Specular'
            )
        else:
            self.layout.label("Object has no material!", icon="MATERIAL")

        self.selfillum_template()

        wire_row = self.layout.row()
        wire_row.prop(
            self.obj.t3d_properties, 'wire',
            icon='WIRE',
            text='Wire',
        )
        wire_row.prop(
            self.obj.t3d_properties, 'wire_size',
            text='WireSize'
        )
        self.layout.row().prop(
            self.obj.t3d_properties, 'opacity',
            icon='RETOPO'
        )
        mapping = self.layout.box()
        mapping.row().prop(
            self.obj.t3d_properties, 'replacableskin_id',
            icon='OUTLINER_OB_SURFACE'
        )



        tex_row = mapping.row()
        if self.obj.t3d_properties.replacableskin_id != "None":
            tex_row.enabled = False
        if self.obj.active_material:
            tex = self.obj.active_material.active_texture
            if tex:
                if tex.image:
                    tex_row.prop(
                        tex.image, "filepath",
                        text='Map')
                else:
                    mapping.label("Texture has not image pinned!", icon="IMAGE_COL")
            else:
                mapping.label("Material has not texture slots!", icon="TEXTURE")
        else:
            mapping.label("Object has no material!", icon="MATERIAL")

    def draw_spot_properties(self):
        self.layout.row().prop(
            self.obj.data, 'color',
            text='Diffuse'
        )

        self.selfillum_template()

        near_atten_box = self.layout.box()

        near_atten_start_row = near_atten_box.row()
        near_atten_start_row.prop(
            self.obj.t3d_properties, "near_atten_start",
            text="NearAttenStart"
        )
        near_atten_end_row = near_atten_box.row()
        near_atten_end_row.prop(
            self.obj.t3d_properties, "near_atten_end",
            text="NearAttenEnd"
        )
        near_atten_box.row().prop(
            self.obj.t3d_properties, "use_near_atten",
            text="UseNearAtten"
        )
        if not self.obj.t3d_properties.use_near_atten:
            near_atten_end_row.enabled = False
            near_atten_start_row.enabled = False

        self.layout.row().prop(
            self.obj.t3d_properties, "far_atten_decay_type",
            "FarAttenDecayType"
        )

        self.layout.row().prop(
            self.obj.data, "distance",
            "FarDecayRadius"
        )

    def draw_stars_properties(self):
        self.layout.row().prop(
            self.obj.t3d_properties, "diffuse_color",
            text="Diffuse"
        )

        self.selfillum_template()

    def selfillum_template(self):
        box = self.layout.box()

        numeric_row = box.row()
        numeric_row.prop(
            self.obj.t3d_properties, "selfillum",
        )

        checkboxes = box.row()
        sa_col = checkboxes.column()
        sa_col.prop(
            self.obj.t3d_properties, "selfillum_always",
            "True"
        )
        sn_col = checkboxes.column()
        sn_col.prop(
            self.obj.t3d_properties, "selfillum_never",
            "False"
        )

        if self.obj.t3d_properties.selfillum_always:
            sn_col.enabled = False
            numeric_row.enabled = False
        elif self.obj.t3d_properties.selfillum_never:
            sa_col.enabled = False
            numeric_row.enabled = False



class AddonPreferences(bpy.types.AddonPreferences):
    bl_idname = __name__

    root_path = StringProperty(
        name='MaSzyna root',
        description='Path to MaSzyna root (main directory)',
        default='',
        subtype='DIR_PATH'
    )

    def draw(self, context):
        self.layout.prop(self, 'root_path')


def register():
    bpy.utils.register_module(__name__)
    bpy.types.Object.t3d_properties = PointerProperty(type=T3DProperties)


def unregister():
    bpy.utils.unregister_module(__name__)

if __name__ == '__main__':
    register()
